package com.unione.handheldscandemo;

import android.content.IntentFilter;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

import com.bigkoo.pickerview.builder.OptionsPickerBuilder;
import com.bigkoo.pickerview.view.OptionsPickerView;
import com.contrarywind.view.WheelView;
import com.tech.hidsdk.BarcodeServiceManager;

import java.util.ArrayList;
import java.util.List;

/**
 * @author bing.guang
 */
public class MainActivity extends AppCompatActivity {
    private static final String TAG = "";

    private Button mSuffixConfirmButton, mScanButton, mTimeoutButton;
    private EditText mScanResult, mSuffixEdit;
    private TextView mScanTimeoutTextView;
    private RadioButton mRbEnter, mRbTab, mRbSpace, mRbNothing;
    private int mTerminalMode = 0;
    private RadioButton mRbInput, mRbKeyevent, mRbBroadcast;
    private int mOutputMode;
    private int mTimeoutIndex;
    private OptionsPickerView mTimeoutPickerView;
    private List<String> mTimeoutList = new ArrayList<>();

    private ScanReceiver mScanReceiver;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.layout_main);
        mSuffixConfirmButton = findViewById(R.id.bt_suffix_confirm);
        mSuffixEdit = findViewById(R.id.et_setting);
        mScanButton = findViewById(R.id.bt_scan);
        mScanResult = findViewById(R.id.et_result);
        mRbInput = findViewById(R.id.rb_input);
        mRbKeyevent = findViewById(R.id.rb_keyevent);
        mRbBroadcast = findViewById(R.id.rb_broadcast);
        mRbEnter = findViewById(R.id.rb_enter);
        mRbTab = findViewById(R.id.rb_tab);
        mRbSpace = findViewById(R.id.rb_space);
        mRbNothing = findViewById(R.id.rb_none);
        mScanResult.setFocusable(true);
        mScanResult.setFocusableInTouchMode(true);
        mScanResult.requestFocus();
        mTimeoutButton = findViewById(R.id.scan_timeout_setting);
        mScanTimeoutTextView = findViewById(R.id.tv_timeout);

        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(ScanReceiver.SCAN_RESULT);
        mScanReceiver = new ScanReceiver();
        registerReceiver(mScanReceiver, intentFilter);

        initActionBar();

        /**
         * bind must be done first.
         */
        BarcodeServiceManager.getInstance().bind(this, null);

        mSuffixConfirmButton.setOnClickListener(view -> {
            /**
             * set suffix
             */
            BarcodeServiceManager.getInstance().setSuffix(mSuffixEdit.getText().toString());
        });

        mScanButton.setOnClickListener(view -> {
            /**
             * start scan
             */
            BarcodeServiceManager.getInstance().startScan();
        });

        mTimeoutList.add("5");
        mTimeoutList.add("10");
        mTimeoutList.add("20");
        mTimeoutList.add("30");
        mTimeoutIndex = SPUtils.getInstance(this).getInt("scan_timeout", 3);
        mScanTimeoutTextView.setText(mTimeoutList.get(mTimeoutIndex) + "s");

        mTimeoutPickerView = new OptionsPickerBuilder(this, (options1, options2, options3, v) -> {
            SPUtils.getInstance(this).put("scan_timeout", options1);
            mScanTimeoutTextView.setText(mTimeoutList.get(options1) + "s");
            mTimeoutIndex = options1;
            /**
             * set timeout cmd
             */
            BarcodeServiceManager.getInstance().setTimeoutCmd(mTimeoutList.get(options1));
        })
                .setCancelColor(Color.BLUE)
                .setSubmitColor(Color.BLUE)
                .setCancelText("Cancel")
                .setSubmitText("Confirm")
                .setDividerType(WheelView.DividerType.FILL)
                .setContentTextSize(20)
                .setOptionsSelectChangeListener((options1, options2, options3) -> {
                })
                .build();
        mTimeoutPickerView.setPicker(mTimeoutList);
        mTimeoutPickerView.setSelectOptions(mTimeoutIndex);

        mTimeoutButton.setOnClickListener(view -> {
            mTimeoutPickerView.show();
        });

        initTerminalMode();
        initOutputMode();
    }

    private void initTerminalMode() {
        mTerminalMode = SPUtils.getInstance(this).getInt("scan_terminal_mode", 0);
        mRbEnter.setChecked(mTerminalMode == 0);
        mRbTab.setChecked(mTerminalMode == 1);
        mRbSpace.setChecked(mTerminalMode == 2);
        mRbNothing.setChecked(mTerminalMode == 3);
        mRbEnter.setOnCheckedChangeListener((buttonView, isChecked) -> setTerminalType(isChecked, terminalType.enter));
        mRbTab.setOnCheckedChangeListener((buttonView, isChecked) -> setTerminalType(isChecked, terminalType.tab));
        mRbSpace.setOnCheckedChangeListener((buttonView, isChecked) -> setTerminalType(isChecked, terminalType.space));
        mRbNothing.setOnCheckedChangeListener((buttonView, isChecked) -> setTerminalType(isChecked, terminalType.nothing));
    }

    private enum terminalType {
        enter, tab, space, nothing
    }

    private void setTerminalType(boolean isChecked, terminalType terminalType) {
        if (isChecked) {
            /**
             * 0 -> Enter
             * 1 -> Tab
             * 2 -> Space
             * 3 -> Nothing
             */
            switch (terminalType) {
                case enter:
                    mRbTab.setChecked(false);
                    mRbSpace.setChecked(false);
                    mRbNothing.setChecked(false);
                    /**
                     * set terminal mode
                     */
                    BarcodeServiceManager.getInstance().setTerminalMode(0);
                    SPUtils.getInstance(this).put("scan_terminal_mode", 0);
                    break;
                case tab:
                    mRbEnter.setChecked(false);
                    mRbSpace.setChecked(false);
                    mRbNothing.setChecked(false);
                    /**
                     * set terminal mode
                     */
                    BarcodeServiceManager.getInstance().setTerminalMode(1);
                    SPUtils.getInstance(this).put("scan_terminal_mode", 1);
                    break;
                case space:
                    mRbEnter.setChecked(false);
                    mRbTab.setChecked(false);
                    mRbNothing.setChecked(false);
                    /**
                     * set terminal mode
                     */
                    BarcodeServiceManager.getInstance().setTerminalMode(2);
                    SPUtils.getInstance(this).put("scan_terminal_mode", 2);
                    break;
                case nothing:
                    mRbEnter.setChecked(false);
                    mRbTab.setChecked(false);
                    mRbSpace.setChecked(false);
                    /**
                     * set terminal mode
                     */
                    BarcodeServiceManager.getInstance().setTerminalMode(3);
                    SPUtils.getInstance(this).put("scan_terminal_mode", 3);
                    break;
            }
        }
    }

    private void initOutputMode() {
        mOutputMode = SPUtils.getInstance(this).getInt("scan_output_mode", 0);
        mRbInput.setChecked(mOutputMode == 0);
        mRbKeyevent.setChecked(mOutputMode == 1);
        mRbBroadcast.setChecked(mOutputMode == 2);
        mRbInput.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                setOutputMode(0);
            }
        });
        mRbKeyevent.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                setOutputMode(1);
            }
        });
        mRbBroadcast.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                setOutputMode(2);
            }
        });
    }

    private void setOutputMode(int mode) {
        mOutputMode = mode;
        mRbInput.setChecked(mOutputMode == 0);
        mRbKeyevent.setChecked(mOutputMode == 1);
        mRbBroadcast.setChecked(mOutputMode == 2);
        /**
         * set output mode
         * 0 means input; 1 means keyevent; 2 means broadcast
         */
        BarcodeServiceManager.getInstance().setOutputMode(mOutputMode);
        SPUtils.getInstance(this).put("scan_output_mode", mOutputMode);
    }

    private void initActionBar() {
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
            actionBar.setCustomView(R.layout.layout_actionbar);

            Button reset = actionBar.getCustomView().findViewById(R.id.tv_reset);
            reset.setOnClickListener(v -> {
                /**
                 * reset
                 */
                BarcodeServiceManager.getInstance().reset();
                resetTerminalMode();
                resetTimeoutMode();
                resetOutputMode();
                resetInnerEdit();
            });
        }
    }

    private void resetInnerEdit() {
        mScanResult.getText().clear();
        mSuffixEdit.getText().clear();
        mScanResult.setFocusable(true);
        mScanResult.requestFocus();
    }

    private void resetOutputMode() {
        SPUtils.getInstance(this).put("scan_output_mode", 0);
        mRbKeyevent.setChecked(false);
        mRbInput.setChecked(true);
        mRbBroadcast.setChecked(false);
        BarcodeServiceManager.getInstance().setOutputMode(0);
    }

    private void resetTimeoutMode() {
        mTimeoutIndex = 3;
        SPUtils.getInstance(this).put("scan_timeout", mTimeoutIndex);
        mScanTimeoutTextView.setText(mTimeoutList.get(mTimeoutIndex) + "s");
        mTimeoutPickerView.setSelectOptions(mTimeoutIndex);
        BarcodeServiceManager.getInstance().setTimeoutCmd(mTimeoutList.get(mTimeoutIndex));
    }

    private void resetTerminalMode() {
        mRbEnter.setChecked(true);
        setTerminalType(true, terminalType.enter);
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy..");
        unregisterReceiver(mScanReceiver);
        /**
         * unbind
         */
        BarcodeServiceManager.getInstance().unBind();
        super.onDestroy();
    }

}

